package fishrungames.yelpmapapp

import android.location.Location
import android.os.Bundle
import android.os.Handler
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.model.Marker
import com.google.maps.android.clustering.Cluster
import java.util.*
import com.google.maps.android.clustering.ClusterManager
import fishrungames.yelpmapapp.google.MapsActivity
import fishrungames.yelpmapapp.records.BusinessDetailRecord
import fishrungames.yelpmapapp.records.MapMarkerRecord
import fishrungames.yelpmapapp.records.ReviewListRecord


/**
 * Created by mephi on 09.06.2017.
 */


class YelpMapActivity : MapsActivity(), ClusterManager.OnClusterItemInfoWindowClickListener<MapMarkerClusterItem>, ClusterManager.OnClusterItemClickListener<MapMarkerClusterItem>, GoogleMap.OnInfoWindowClickListener
{


    val dataProvider = DataProvider(this,
            this::handleMarkersUpdated,
            this::handleUpdateBusinessDetail,
            this::handleUpdateReviewList)

    var yelpInfoWindowProvider : YelpInfoWindowProvider? = null

    private var mClusterManager: ClusterManager<MapMarkerClusterItem>? = null

    val markerMap = mutableMapOf<String, MapMarkerClusterItem>()

    var lastClickedMapMarker : MapMarkerClusterItem? = null

    override fun onMapReady(googleMap: GoogleMap) {
        super.onMapReady(googleMap)

        googleMap.setOnCameraMoveListener {

            dataProvider.requestMapMarkerData(googleMap.cameraPosition.target.latitude, googleMap.cameraPosition.target.longitude)

        }

        googleMap.setOnInfoWindowClickListener(this)

        //yelpInfoWindowProvider = YelpInfoWindowProvider(this)

        //googleMap.setInfoWindowAdapter(yelpInfoWindowProvider)

        setUpClusterer(googleMap)
    }


    override fun onLocationChanged(location: Location) {
        super.onLocationChanged(location)
        dataProvider.requestMapMarkerData(location.latitude, location.longitude)
    }

    fun handleMarkersUpdated(newMarkerMap: Map<String, MapMarkerRecord>)
    {
        if (mClusterManager == null)
        {
            return
        }

        val clusterManager = mClusterManager!!

        for (key in newMarkerMap.keys)
        {


            if (!markerMap.containsKey(key))
            {
                val mapMarkerRecord = newMarkerMap[key] as MapMarkerRecord

                if (mapMarkerRecord.coordinates != null && mapMarkerRecord.coordinates.latitude != null && mapMarkerRecord.coordinates.longitude != null ) {

                    val mapMarkerClusterItem = MapMarkerClusterItem(mapMarkerRecord)


                    clusterManager.addItem(mapMarkerClusterItem)

                    markerMap[key] = mapMarkerClusterItem
                }

            }
        }
    }


    private fun setUpClusterer(googleMap: GoogleMap) {


        mClusterManager = ClusterManager<MapMarkerClusterItem>(this, googleMap)

        googleMap.setOnCameraIdleListener(mClusterManager)
        googleMap.setOnMarkerClickListener(mClusterManager)

        googleMap.setInfoWindowAdapter(mClusterManager?.markerManager)


        yelpInfoWindowProvider = YelpInfoWindowProvider(this)

        mClusterManager?.markerCollection?.setOnInfoWindowAdapter(yelpInfoWindowProvider)

        //mClusterManager?.markerCollection?.setOnInfoWindowClickListener(this)

        //mClusterManager?.setOnClusterItemInfoWindowClickListener(this)

        mClusterManager?.setOnClusterItemClickListener(this)

    }


    override fun onInfoWindowClick(item: Marker?) {

        if (lastClickedMapMarker != null)
        {
            val detailFragment = DetailFragment()

            val bundle = Bundle()

            bundle.putSerializable("mapMarkerRecord", lastClickedMapMarker!!.mapMarkerRecord)

            detailFragment.arguments = bundle

            supportFragmentManager.beginTransaction().add(R.id.container, detailFragment, "detailFragment").addToBackStack("detailFragment").commit()

        }

    }

    override fun onClusterItemInfoWindowClick(item: MapMarkerClusterItem?) {

        if (item != null) {

            val detailFragment = DetailFragment()

            val bundle = Bundle()

            bundle.putSerializable("mapMarkerRecord", item.mapMarkerRecord)

            detailFragment.arguments = bundle

            supportFragmentManager.beginTransaction().add(R.id.container, detailFragment, "detailFragment").addToBackStack("detailFragment").commit()

        }
    }

    override fun onClusterItemClick(item: MapMarkerClusterItem): Boolean {
        // Does nothing, but you could go into the user's profile page, for example.

        lastClickedMapMarker = item

        return false
    }

    fun handleUpdateBusinessDetail(businessDetailRecord : BusinessDetailRecord)
    {
        val detailFragment = supportFragmentManager.findFragmentByTag("detailFragment") as? DetailFragment

        detailFragment?.updateBusinessDetail(businessDetailRecord)
    }

    fun handleUpdateReviewList(reviewListRecordPair : Pair<String, ReviewListRecord>)
    {
        val detailFragment = supportFragmentManager.findFragmentByTag("detailFragment") as? DetailFragment

        detailFragment?.updateReviewList(reviewListRecordPair)

    }

}